/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：转诊记录
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.rest;

import java.util.*;

import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.xwd.hospital.server.annotation.ApiCommon;
import com.xwd.hospital.server.annotation.AuthUser;
import com.xwd.hospital.server.base.BaseController;
import com.xwd.hospital.server.domain.DoctorInfo;
import com.xwd.hospital.server.domain.User;
import com.xwd.hospital.server.dto.ReferralDoctorDto;
import com.xwd.hospital.server.enums.AuditStateEnum;
import com.xwd.hospital.server.rest.res.ApiCode;
import com.xwd.hospital.server.domain.ReferralRecord;
import com.xwd.hospital.server.rest.req.ReferralRecordParam;
import com.xwd.hospital.server.rest.res.ApiResponse;
import com.xwd.hospital.server.service.DoctorInfoService;
import com.xwd.hospital.server.service.ReferralRecordService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "ReferralRecordController", description = "转诊记录管理")
@RequestMapping("/v1/pms/referral-record")
public class ReferralRecordController extends BaseController<ReferralRecordService, ReferralRecord, ReferralRecordParam> {

    @Resource
    private DoctorInfoService doctorInfoService;

    @Resource
    public void setService(ReferralRecordService service) {
        this.service = service;
    }

    /**
     * 新增
     *
     * @param entity 要新增的对象
     * @return 新增的对象
     */
    @PostMapping("/")
    @Operation(summary = "新增")
    @Override
    public ApiResponse<ReferralRecord> add(@RequestBody ReferralRecord entity, @AuthUser User user) {
        entity.setEditorId(user.getId());
        entity.setEditorName(user.getUsername());

        DoctorInfo doctorInfo = doctorInfoService.lambdaQuery().eq(DoctorInfo::getUserId, user.getId()).one();
        if(Objects.isNull(doctorInfo)){
            throw new ApiCode.ApiException(-1,"信息错误");

        }
        if(entity.getOutDoctorId() == entity.getInDoctorId()){
            throw new ApiCode.ApiException(-1,"信息错误");
        }
        entity.setDoctorId(doctorInfo.getId());
        entity.setDefault().validate(true);
        this.service.save(entity);
        return ApiResponse.ok(this.service.getById(entity.getId()));
    }

    /**
     * 批量新增
     *
     * @param entityList 要新增的对象
     * @return boolean 成功或失败
     */
    @PostMapping("/batch-save")
    @Operation(summary = "批量新增")
    @Override
    public ApiResponse<Boolean> batchSave(@RequestBody List<ReferralRecord> entityList, @AuthUser User user) {
        entityList.forEach(entity -> {
            entity.setEditorId(user.getId());
            entity.setEditorName(user.getUsername());
            entity.setDefault().validate(true);
        });
        return ApiResponse.ok(this.service.saveBatch(entityList));
    }

    @Override
    public ApiResponse<IPage<ReferralRecord>> selectByPage(ReferralRecordParam param,
                                                           @RequestParam(value = "orderByClause", defaultValue = "id desc") String orderByClause,
                                                           @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                           @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {

        IPage<ReferralRecord> page = new Page<>(pageNum, pageSize);
        if(null != param.getDoctorId()){
            QueryWrapper<ReferralRecord> wrapper  = new QueryWrapper<ReferralRecord>()
                    .eq("out_doctor_id", param.getDoctorId())
                    .or()
                    .eq("in_doctor_id", param.getDoctorId())
                    .eq("audit_state", AuditStateEnum.AUDIT_SUCCESS)
                    .orderByDesc("create_time");
            return ApiResponse.ok(this.service.page(page, wrapper));
        }else {
            QueryWrapper<ReferralRecord> wrapper = param.toQueryWrapper();
            String[] temp = orderByClause.split("[,;]");
            Arrays.stream(temp).forEach(ob -> {
                String[] obTemp = ob.split("\\s");
                boolean isAsc = obTemp.length == 1 || obTemp[1].equalsIgnoreCase("asc");
                wrapper.orderBy(true, isAsc, obTemp[0]);
            });
            return ApiResponse.ok(this.service.page(page, wrapper));
        }

    }

    /**
     * 审核
     */
    @PostMapping("/audit")
    @Operation(summary = "审核")
    public ApiResponse<Boolean> audit(@RequestBody ReferralRecord entity, @AuthUser User user) {
        return ApiResponse.ok(this.service.audit(entity));
    }

    /**
     * 用户查询转诊记录
     */
    @PostMapping("/listMyRecord/{userInfoId}")
    @Operation(summary = "用户查询转诊记录")
    public ApiResponse listMyRecord(@PathVariable("userInfoId") Long userInfoId,
                                    @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                    @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        Page<ReferralRecord> page = new Page<>(pageNum, pageSize);
        return ApiResponse.ok(this.service.listMyRecord(page,userInfoId));
    }


}
